@@ -4,19 +4,12 @@
 
 use anyhow::Result;
 use serde::Serialize;
-use subplot::{DataFile, Document, Style, SubplotError};
-
 use std::convert::TryFrom;
+use std::fmt::Debug;
 use std::path::Path;
 use std::str::FromStr;
-
-pub fn load_document<P: AsRef<Path>>(filename: P, style: Style) -> Result<Document> {
-    let filename = filename.as_ref();
-    let base_path = subplot::get_basedir_from(filename)?;
-    let doc = Document::from_file(&base_path, filename, style)?;
-
-    Ok(doc)
-}
+use subplot::{DataFile, Document, Style, SubplotError};
+use tracing::{event, instrument, Level};
 
 pub fn extract_file<'a>(doc: &'a Document, filename: &str) -> Result<&'a DataFile> {
     for file in doc.files() {
@@ -52,21 +45,21 @@ impl TryFrom<&mut Document> for Metadata {
             .meta()
             .bindings_filenames()
             .into_iter()
-            .map(|p| filename(Some(&p)))
+            .map(|p| filename(Some(p)))
             .collect();
         binding_files.sort_unstable();
         let mut function_files: Vec<_> = doc
             .meta()
             .functions_filenames()
             .into_iter()
-            .map(|p| filename(Some(&p)))
+            .map(|p| filename(Some(p)))
             .collect();
         function_files.sort_unstable();
         let mut bibliographies: Vec<_> = doc
             .meta()
             .bibliographies()
             .into_iter()
-            .map(|p| filename(Some(&p)))
+            .map(|p| filename(Some(p)))
             .collect();
         bibliographies.sort_unstable();
         let mut scenarios: Vec<_> = doc
