@@ -5,9 +5,8 @@ use anyhow::Result;
 
 use chrono::{Local, TimeZone};
 use structopt::StructOpt;
-use subplot::{
-    generate_test_program, resource, template_spec, DataFile, Document, MarkupOpts, Style,
-};
+use subplot::{codegen, load_document, resource, DataFile, Document, MarkupOpts, Style};
+use tracing::{event, instrument, span, Level, Subscriber};
 
 use std::convert::TryFrom;
 use std::ffi::OsString;
@@ -54,6 +53,7 @@ enum Cmd {
     Metadata(Metadata),
     Docgen(Docgen),
     Codegen(Codegen),
+    Resources(Resources),
 }
 
 impl Cmd {
@@ -64,6 +64,7 @@ impl Cmd {
             Cmd::Metadata(m) => m.run(),
             Cmd::Docgen(d) => d.run(),
             Cmd::Codegen(c) => c.run(),
+            Cmd::Resources(r) => r.run(),
         }
     }
 
@@ -74,6 +75,7 @@ impl Cmd {
             Cmd::Metadata(m) => m.doc_path(),
             Cmd::Docgen(d) => d.doc_path(),
             Cmd::Codegen(c) => c.doc_path(),
+            Cmd::Resources(r) => r.doc_path(),
         }
     }
 }
@@ -105,6 +107,22 @@ fn long_version() -> Result<String> {
     Ok(ret)
 }
 
+#[derive(Debug, StructOpt)]
+/// Examine embedded resources built into Subplot
+#[structopt(setting = structopt::clap::AppSettings::Hidden)]
+struct Resources {}
+
+impl Resources {
+    fn doc_path(&self) -> Option<&Path> {
+        None
+    }
+    fn run(&self) -> Result<()> {
+        for (name, bytes) in subplot::resource::embedded_files() {
+            println!("{} {} bytes", name, bytes.len());
+        }
+        Ok(())
+    }
+}
 #[derive(Debug, StructOpt)]
 /// Extract embedded files from a subplot document
 ///
@@ -140,7 +158,9 @@ impl Extract {
     }
 
     fn run(&self) -> Result<()> {
-        let doc = cli::load_document(&self.filename, Style::default())?;
+        let span = span!(Level::TRACE, "extract");
+        let _enter = span.enter();
+        let doc = load_document(&self.filename, Style::default())?;
 
         let files: Vec<&DataFile> = if self.embedded.is_empty() {
             doc.files()
@@ -194,6 +214,8 @@ impl Filter {
     }
 
     fn run(&self) -> Result<()> {
+        let span = span!(Level::TRACE, "filter");
+        let _enter = span.enter();
         let mut buffer = String::new();
         if let Some(filename) = &self.input {
             File::open(filename)?.read_to_string(&mut buffer)?;
@@ -242,7 +264,9 @@ impl Metadata {
     }
 
     fn run(&self) -> Result<()> {
-        let mut doc = cli::load_document(&self.filename, Style::default())?;
+        let span = span!(Level::TRACE, "metadata");
+        let _enter = span.enter();
+        let mut doc = load_document(&self.filename, Style::default())?;
         let meta = cli::Metadata::try_from(&mut doc)?;
         match self.output_format {
             cli::OutputFormat::Plain => meta.write_out(),
@@ -275,14 +299,25 @@ impl Docgen {
         self.input.parent()
     }
 
+    #[instrument(level = "trace", skip(self))]
     fn run(&self) -> Result<()> {
+        let span = span!(Level::TRACE, "docgen");
+        let _enter = span.enter();
         let mut style = Style::default();
         if self.output.extension() == Some(&OsString::from("pdf")) {
+            event!(Level::TRACE, "PDF output chosen");
             style.typeset_links_as_notes();
         }
-        let mut doc = cli::load_document(&self.input, style)?;
+        let mut doc = load_document(&self.input, style)?;
+        event!(Level::TRACE, "Got doc, now linting it");
         doc.lint()?;
-        if !doc.check_named_files_exist()? {
+        event!(Level::TRACE, "Doc linted ok");
+        let meta = doc.meta();
+        event!(Level::TRACE, ?meta, "Looking for template");
+        let template = meta.template_name().unwrap_or("").to_string();
+        event!(Level::TRACE, ?template);
+        if !doc.check_named_files_exist(&template)? || !doc.check_matched_steps_have_impl(&template)
+        {
             eprintln!("Continuing despite warnings");
         }
 
@@ -376,22 +411,16 @@ impl Codegen {
     }
 
     fn run(&self) -> Result<()> {
-        let mut doc = cli::load_document(&self.filename, Style::default())?;
-        doc.lint()?;
-        if !doc.check_named_files_exist()? {
-            eprintln!("Unable to continue");
-            std::process::exit(1);
-        }
-
-        let spec = template_spec(&doc)?;
-        generate_test_program(&mut doc, &spec, &self.output)?;
+        let span = span!(Level::TRACE, "codegen");
+        let _enter = span.enter();
 
+        let output = codegen(&self.filename, &self.output)?;
         if self.run {
-            let run = match spec.run() {
+            let run = match output.spec.run() {
                 None => {
                     eprintln!(
                         "Template {} does not specify how to run suites",
-                        spec.template_filename().display()
+                        output.spec.template_filename().display()
                     );
                     std::process::exit(1);
                 }
@@ -408,7 +437,8 @@ impl Codegen {
     }
 }
 
-fn main() {
+fn real_main() {
+    event!(Level::TRACE, "Starting Subplot");
     let argparser = Toplevel::clap();
     let version = long_version().unwrap();
     let argparser = argparser.long_version(version.as_str());
@@ -416,10 +446,77 @@ fn main() {
     let args = Toplevel::from_clap(&args);
     args.handle_special_args();
     match args.run() {
-        Ok(_) => {}
+        Ok(_) => {
+            event!(Level::TRACE, "Finished successfully");
+        }
         Err(e) => {
+            event!(Level::DEBUG, error = ?e, "Failed: {:?}", e);
             eprintln!("Failure: {:?}", e);
             process::exit(1);
         }
     }
 }
+
+enum LogFormat {
+    Pretty,
+    OneLine,
+    Json,
+}
+
+fn main() {
+    let filter = match tracing_subscriber::EnvFilter::try_from_env("SUBPLOT_LOG") {
+        Ok(f) => f,
+        Err(e) => {
+            if std::env::var_os("SUBPLOT_LOG").is_none() {
+                // Missing SUBPLOT_LOG, default filter should be INFO
+                tracing_subscriber::EnvFilter::new("info")
+            } else {
+                eprintln!("Unable to parse log directive from SUBPLOT_LOG: {:?}", e);
+                process::exit(1);
+            }
+        }
+    };
+    let format = match std::env::var("SUBPLOT_LOG_FORMAT")
+        .map(|v| v.to_ascii_lowercase())
+        .as_deref()
+    {
+        Err(_) | Ok("oneline") | Ok("default") => LogFormat::OneLine,
+        Ok("pretty") => LogFormat::Pretty,
+        Ok("json") => LogFormat::Json,
+        Ok(v) => {
+            eprintln!("Unknown log format: {:?}", v);
+            process::exit(1);
+        }
+    };
+    match std::env::var("SUBPLOT_LOG_FILE").ok().as_deref() {
+        None | Some("") | Some("-") => {
+            let subscriber = tracing_subscriber::fmt().with_env_filter(filter);
+            let subscriber: Box<dyn Subscriber + Send + Sync + 'static> = match format {
+                LogFormat::OneLine => Box::new(subscriber.finish()),
+                LogFormat::Pretty => Box::new(subscriber.pretty().finish()),
+                LogFormat::Json => Box::new(subscriber.json().with_ansi(false).finish()),
+            };
+            tracing::subscriber::with_default(subscriber, real_main);
+        }
+        Some(fname) => {
+            let curdir = std::path::Component::CurDir;
+            let location = PathBuf::from(fname);
+            let parent = location.parent().unwrap_or_else(|| curdir.as_ref());
+            let fname = location
+                .file_name()
+                .expect("No filename given in SUBPLOT_TRACE_LOG");
+            let appender = tracing_appender::rolling::never(parent, fname);
+            let (non_blocking, _guard) = tracing_appender::non_blocking(appender);
+            let subscriber = tracing_subscriber::fmt()
+                .with_writer(non_blocking)
+                .with_env_filter(filter)
+                .with_ansi(false);
+            let subscriber: Box<dyn Subscriber + Send + Sync + 'static> = match format {
+                LogFormat::OneLine => Box::new(subscriber.finish()),
+                LogFormat::Pretty => Box::new(subscriber.pretty().finish()),
+                LogFormat::Json => Box::new(subscriber.json().finish()),
+            };
+            tracing::subscriber::with_default(subscriber, real_main);
+        }
+    }
+}
