@@ -1,3 +1,6 @@
+use crate::ast;
+use crate::generate_test_program;
+use crate::get_basedir_from;
 use crate::visitor;
 use crate::DataFile;
 use crate::DataFiles;
@@ -9,15 +12,20 @@ use crate::Scenario;
 use crate::ScenarioStep;
 use crate::Style;
 use crate::{bindings::CaptureType, parser::parse_scenario_snippet};
+use crate::{template_spec, TemplateSpec};
 use crate::{Result, SubplotError};
 
 use std::collections::HashSet;
 use std::default::Default;
+use std::fmt::Debug;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
+use std::str::FromStr;
 
 use pandoc_ast::{MutVisitor, Pandoc};
 
+use tracing::{event, instrument, span, Level};
+
 /// The set of known (special) classes which subplot will always recognise
 /// as being valid.
 static SPECIAL_CLASSES: &[&str] = &["scenario", "file", "dot", "pikchr", "plantuml", "roadmap"];
@@ -95,7 +103,32 @@ impl<'a> Document {
         }
     }
 
+    #[instrument(level = "trace", skip(ast))]
+    fn from_ast<P>(
+        basedir: P,
+        markdowns: Vec<PathBuf>,
+        mut ast: Pandoc,
+        style: Style,
+    ) -> Result<Document>
+    where
+        P: AsRef<Path> + Debug,
+    {
+        let meta = Metadata::new(basedir, &ast)?;
+        let mut linter = LintingVisitor::default();
+        event!(Level::TRACE, "Walking AST for linting...");
+        linter.walk_pandoc(&mut ast);
+        if !linter.issues.is_empty() {
+            // Currently we can't really return more than one error so return one
+            return Err(linter.issues.remove(0));
+        }
+        let files = DataFiles::new(&mut ast);
+        let doc = Document::new(markdowns, ast, meta, files, style);
+        event!(Level::TRACE, "Loaded from JSON OK");
+        Ok(doc)
+    }
+
     /// Construct a Document from a JSON AST
+    #[instrument(level = "trace", skip(json))]
     pub fn from_json<P>(
         basedir: P,
         markdowns: Vec<PathBuf>,
@@ -103,18 +136,11 @@ impl<'a> Document {
         style: Style,
     ) -> Result<Document>
     where
-        P: AsRef<Path>,
+        P: AsRef<Path> + Debug,
     {
-        let mut ast: Pandoc = serde_json::from_str(json)?;
-        let meta = Metadata::new(basedir, &ast)?;
-        let mut linter = LintingVisitor::default();
-        linter.walk_pandoc(&mut ast);
-        if !linter.issues.is_empty() {
-            // Currently we can't really return more than one error so return one
-            return Err(linter.issues.remove(0));
-        }
-        let files = DataFiles::new(&mut ast);
-        Ok(Document::new(markdowns, ast, meta, files, style))
+        event!(Level::TRACE, "Parsing document...");
+        let ast: Pandoc = serde_json::from_str(json)?;
+        Self::from_ast(basedir, markdowns, ast, style)
     }
 
     /// Construct a Document from a named file.
@@ -122,6 +148,7 @@ impl<'a> Document {
     /// The file can be in any format Pandoc understands. This runs
     /// Pandoc to parse the file into an AST, so it can be a little
     /// slow.
+    #[instrument(level = "trace")]
     pub fn from_file(basedir: &Path, filename: &Path, style: Style) -> Result<Document> {
         let markdowns = vec![filename.to_path_buf()];
 
@@ -135,17 +162,42 @@ impl<'a> Document {
         pandoc.set_output(pandoc::OutputKind::Pipe);
 
         // Add external Pandoc filters.
-        let citeproc = std::path::Path::new("pandoc-citeproc");
-        pandoc.add_option(pandoc::PandocOption::Filter(citeproc.to_path_buf()));
+        crate::policy::add_citeproc(&mut pandoc);
 
+        event!(Level::TRACE, ?filename, "Invoking Pandoc to parse document");
         let output = match pandoc.execute()? {
             pandoc::PandocOutput::ToBuffer(o) => o,
             _ => return Err(SubplotError::NotJson),
         };
         let doc = Document::from_json(basedir, markdowns, &output, style)?;
+        event!(Level::TRACE, "Loaded document OK");
         Ok(doc)
     }
 
+    /// Construct a Document from a named file, using the pullmark_cmark crate.
+    ///
+    /// The file can be in the CommonMark format, with some
+    /// extensions. This uses the pulldown-cmark crate to parse the
+    /// file into an AST.
+    #[instrument(level = "trace")]
+    pub fn from_file_with_pullmark(
+        basedir: &Path,
+        filename: &Path,
+        style: Style,
+    ) -> Result<Document> {
+        event!(
+            Level::TRACE,
+            ?filename,
+            "Parsing document with pullmark-cmark"
+        );
+        let filename = filename.to_path_buf();
+        let markdown = std::fs::read_to_string(&filename)?;
+        let ast = ast::AbstractSyntaxTree::from_str(&markdown)?;
+
+        event!(Level::TRACE, "Parsed document OK");
+        Self::from_ast(basedir, vec![filename], ast.to_pandoc(), style)
+    }
+
     /// Return the AST of a Document, serialized as JSON.
     ///
     /// This is useful in a Pandoc filter, so that the filter can give
@@ -198,10 +250,13 @@ impl<'a> Document {
     }
 
     /// Check the document for common problems.
+    #[instrument(level = "trace", skip(self))]
     pub fn lint(&self) -> Result<()> {
+        event!(Level::TRACE, "Linting document");
         self.check_doc_has_title()?;
         self.check_filenames_are_unique()?;
         self.check_block_classes()?;
+        event!(Level::TRACE, "No linting problems found");
         Ok(())
     }
 
@@ -260,14 +315,16 @@ impl<'a> Document {
 
     /// Check that all named files (in matched steps) are actually present in the
     /// document.
-    pub fn check_named_files_exist(&mut self) -> Result<bool> {
+    #[instrument(level = "trace", skip(self))]
+    pub fn check_named_files_exist(&mut self, template: &str) -> Result<bool> {
         let filenames: HashSet<_> = self
             .files()
             .iter()
             .map(|f| f.filename().to_lowercase())
             .collect();
+        event!(Level::TRACE, ?filenames, "Checking that files exist");
         let mut okay = true;
-        let scenarios = match self.matched_scenarios() {
+        let scenarios = match self.matched_scenarios(template) {
             Ok(scenarios) => scenarios,
             Err(_) => return Ok(true), // We can't do the check, so say it's okay.
         };
@@ -288,10 +345,47 @@ impl<'a> Document {
         Ok(okay)
     }
 
+    /// Check that all matched steps actually have function implementations
+    #[instrument(level = "trace", skip(self))]
+    pub fn check_matched_steps_have_impl(&mut self, template: &str) -> bool {
+        event!(Level::TRACE, "Checking that steps have implementations");
+        let mut okay = true;
+        let scenarios = match self.matched_scenarios(template) {
+            Ok(scenarios) => scenarios,
+            Err(_) => return true, // No matches means no missing impls
+        };
+        event!(Level::TRACE, count = scenarios.len(), "Found scenarios");
+        for scenario in scenarios {
+            event!(
+                Level::TRACE,
+                title = scenario.title(),
+                "Checking that steps in scenario"
+            );
+            let mut said_scenario = false;
+            for step in scenario.steps() {
+                if step.function().is_none() {
+                    if !said_scenario {
+                        eprintln!("Scenario: '{}'", scenario.title());
+                        eprintln!("  Template: '{}'", template);
+                        said_scenario = true;
+                    }
+                    eprintln!("  Step missing implementation: '{}'", step.text());
+                    event!(
+                        Level::ERROR,
+                        step = step.text(),
+                        "Missing step implementation"
+                    );
+                    okay = false;
+                }
+            }
+        }
+        okay
+    }
+
     /// Typeset a Subplot document.
     pub fn typeset(&mut self) {
         let mut visitor =
-            visitor::TypesettingVisitor::new(self.style.clone(), &self.meta.bindings());
+            visitor::TypesettingVisitor::new(self.style.clone(), self.meta.bindings());
         visitor.walk_pandoc(&mut self.ast);
     }
 
@@ -314,16 +408,110 @@ impl<'a> Document {
     }
 
     /// Return matched scenarios in a document.
-    pub fn matched_scenarios(&mut self) -> Result<Vec<MatchedScenario>> {
+    #[instrument(skip(self))]
+    pub fn matched_scenarios(&mut self, template: &str) -> Result<Vec<MatchedScenario>> {
         let scenarios = self.scenarios()?;
+        event!(
+            Level::TRACE,
+            "Found {} scenarios, checking their bindings",
+            scenarios.len()
+        );
         let bindings = self.meta().bindings();
         scenarios
             .iter()
-            .map(|scen| MatchedScenario::new(scen, bindings))
+            .map(|scen| MatchedScenario::new(template, scen, bindings))
             .collect()
     }
 }
 
+/// Load a `Document` from a file.
+///
+/// This version uses Pandoc to parse the Markdown.
+#[instrument(level = "trace")]
+pub fn load_document<P>(filename: P, style: Style) -> Result<Document>
+where
+    P: AsRef<Path> + Debug,
+{
+    let filename = filename.as_ref();
+    let base_path = get_basedir_from(filename);
+    event!(
+        Level::TRACE,
+        ?filename,
+        ?base_path,
+        "Loading document based at `{}` called `{}` with {:?}",
+        base_path.display(),
+        filename.display(),
+        style
+    );
+    let doc = Document::from_file(&base_path, filename, style)?;
+    event!(Level::TRACE, "Loaded doc from file OK");
+
+    Ok(doc)
+}
+
+/// Load a `Document` from a file.
+///
+/// This version uses the `cmark-pullmark` crate to parse Markdown.
+#[instrument(level = "trace")]
+pub fn load_document_with_pullmark<P>(filename: P, style: Style) -> Result<Document>
+where
+    P: AsRef<Path> + Debug,
+{
+    let filename = filename.as_ref();
+    let base_path = get_basedir_from(filename);
+    event!(
+        Level::TRACE,
+        ?filename,
+        ?base_path,
+        "Loading document based at `{}` called `{}` with {:?} using pullmark-cmark",
+        base_path.display(),
+        filename.display(),
+        style
+    );
+    crate::resource::add_search_path(filename.parent().unwrap());
+    let doc = Document::from_file_with_pullmark(&base_path, filename, style)?;
+    event!(Level::TRACE, "Loaded doc from file OK");
+    Ok(doc)
+}
+
+/// Generate code for one document.
+pub fn codegen(filename: &Path, output: &Path) -> Result<CodegenOutput> {
+    let span = span!(Level::TRACE, "codegen");
+    let _enter = span.enter();
+
+    let mut doc = load_document_with_pullmark(filename, Style::default())?;
+    doc.lint()?;
+    let template = doc
+        .meta()
+        .template_name()
+        .ok_or(SubplotError::MissingTemplate)?
+        .to_string();
+    event!(Level::TRACE, ?template);
+    if !doc.check_named_files_exist(&template)? || !doc.check_matched_steps_have_impl(&template) {
+        event!(Level::ERROR, "Found problems in document, cannot continue");
+        eprintln!("Unable to continue");
+        std::process::exit(1);
+    }
+
+    event!(Level::TRACE, "Generating code");
+    let spec = template_spec(&doc)?;
+    generate_test_program(&mut doc, &spec, output, &template)?;
+    event!(Level::TRACE, "Finished generating code");
+
+    Ok(CodegenOutput::new(spec, doc))
+}
+
+pub struct CodegenOutput {
+    pub spec: TemplateSpec,
+    pub doc: Document,
+}
+
+impl CodegenOutput {
+    fn new(spec: TemplateSpec, doc: Document) -> Self {
+        Self { spec, doc }
+    }
+}
+
 fn extract_scenario(e: &[visitor::Element]) -> Result<(Option<Scenario>, usize)> {
     if e.is_empty() {
         // If we get here, it's a programming error.
@@ -333,7 +521,7 @@ fn extract_scenario(e: &[visitor::Element]) -> Result<(Option<Scenario>, usize)>
     match &e[0] {
         visitor::Element::Snippet(_) => Err(SubplotError::ScenarioBeforeHeading),
         visitor::Element::Heading(title, level) => {
-            let mut scen = Scenario::new(&title);
+            let mut scen = Scenario::new(title);
             let mut prevkind = None;
             for (i, item) in e.iter().enumerate().skip(1) {
                 match item {
@@ -351,7 +539,7 @@ fn extract_scenario(e: &[visitor::Element]) -> Result<(Option<Scenario>, usize)>
                         }
                     }
                     visitor::Element::Snippet(text) => {
-                        for line in parse_scenario_snippet(&text) {
+                        for line in parse_scenario_snippet(text) {
                             let step = ScenarioStep::new_from_str(line, prevkind)?;
                             scen.add(&step);
                             prevkind = Some(step.kind());
