@@ -40,6 +40,7 @@ pub use metadata::Metadata;
 
 mod doc;
 pub use doc::Document;
+pub use doc::{codegen, load_document, load_document_with_pullmark};
 
 mod style;
 pub use style::Style;
@@ -69,3 +70,6 @@ pub use templatespec::TemplateSpec;
 
 mod codegen;
 pub use codegen::{generate_test_program, template_spec};
+
+mod ast;
+pub use ast::AbstractSyntaxTree;
