@@ -15,18 +15,18 @@ impl MutVisitor for LintingVisitor {
                 Block::CodeBlock(attr, _) => {
                     if panhelper::is_class(attr, "file") {
                         let newlines: Vec<_> =
-                            panhelper::find_attr_kv(&attr, "add-newline").collect();
+                            panhelper::find_attr_kv(attr, "add-newline").collect();
                         match newlines.len() {
                             0 => {}
                             1 => match newlines[0].to_ascii_lowercase().as_ref() {
                                 "auto" | "yes" | "no" => {}
                                 _ => self.issues.push(SubplotError::UnrecognisedAddNewline(
-                                    panhelper::get_filename(&attr),
+                                    panhelper::get_filename(attr),
                                     newlines[0].to_owned(),
                                 )),
                             },
                             _ => self.issues.push(SubplotError::RepeatedAddNewlineAttribute(
-                                panhelper::get_filename(&attr),
+                                panhelper::get_filename(attr),
                             )),
                         }
                     }
